/***************************************************************************
                          oooqs.cpp  -  description
                             -------------------
    begin                : Mon Apr 29 17:48:15 CEST 2002
    copyright            : (C) 2002 by Christian Nitschkowski
    email                : segfault_ii@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "oooqs.h"
#include <klocale.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <kurlrequester.h>
#include <qcheckbox.h>
#include <kprocio.h>
#include <qcursor.h>
#include <qradiobutton.h>
#include <qtooltip.h>
#include <qspinbox.h>

OooQs::OooQs(KDialog* d, QWidget *parent, const char *name) : KDockWindow(parent, name), actionCollection(this), aboutDialog(d)
{
	/** Make sure, we won't access a non-existing object */
	open = 0;
	editMenuFirst = true;

	/** Set the OpenOffice icon */
	KIconLoader* loader = KGlobal::iconLoader();
	icon = loader->loadIcon("oooqs", KIcon::Toolbar);
	setPixmap(icon);
	
	/** Create the preferences-dialog */
	preferencesDialog = new prefDialogImpl(0);
	
	/** Define the actions */
	about = KStdAction::aboutApp(this, SLOT (showAbout()), &actionCollection);
	file = KStdAction::open(this, SLOT (fileOpen()), &actionCollection);
	recent = KStdAction::openRecent(this, SLOT (openRecent(const KURL&)), &actionCollection);
	preferences = KStdAction::preferences (this, SLOT (openPreferences()), &actionCollection);
	icon = loader->loadIcon("ooo_calc", KIcon::Toolbar);
	calc = new KAction(i18n("OpenOffice.org Calc"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_draw", KIcon::Toolbar);
	draw = new KAction(i18n("OpenOffice.org Draw"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_impress", KIcon::Toolbar);
	impress = new KAction(i18n("OpenOffice.org Impress"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_global", KIcon::Toolbar);
	global = new KAction(i18n("OpenOffice.org Global Document"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_math", KIcon::Toolbar);
	math = new KAction(i18n("OpenOffice.org Math"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_web", KIcon::Toolbar);
	web = new KAction(i18n("OpenOffice.org Web"), icon, 0, 0, 0, &actionCollection, 0);
	icon = loader->loadIcon("ooo_writer", KIcon::Toolbar);
	writer = new KAction(i18n("OpenOffice.org Writer"), icon, 0, 0, 0, &actionCollection, 0);
	
	for (int i=0; i < 4; i++){
		sep[i] = new KActionSeparator;
	}

	/** Fill the menu */
/*	file->plug(contextMenu());
	recent->plug(contextMenu());
	contextMenu()->insertSeparator();
	calc->plug(contextMenu());
	draw->plug(contextMenu());
	impress->plug(contextMenu());
	global->plug(contextMenu());
	math->plug(contextMenu());
	web->plug(contextMenu());
	writer->plug(contextMenu());
	contextMenu()->insertSeparator();
	preferences->plug(contextMenu());
	contextMenu()->insertSeparator();
	about->plug(contextMenu());
	sep[0]->plug(contextMenu());
	sep[1]->plug(contextMenu());
	sep[2]->plug(contextMenu());*/

	/** Connect the actions with the slots */
	connect (calc, SIGNAL (activated()), SLOT (startCalc()));
	connect (draw, SIGNAL (activated()), SLOT (startDraw()));
	connect (impress, SIGNAL (activated()), SLOT (startImpress()));
	connect (global, SIGNAL (activated()), SLOT (startGlobal()));
	connect (math, SIGNAL (activated()), SLOT (startMath()));
	connect (web, SIGNAL (activated()), SLOT (startWeb()));
	connect (writer, SIGNAL (activated()), SLOT (startWriter()));
	connect (preferencesDialog, SIGNAL (success()), SLOT (changedConfig()));
	connect (preferencesDialog->clear, SIGNAL (clicked()), this, SLOT (clearURLList()));

	/** Read the configuration */
	config = KGlobal::config();
	
	/** Load the list of recent files */
	recent->loadEntries(config);

	/** Try to find the entries in the configfile */
	/** If the config is not accessible (e.g. not existing) */
	config->setGroup("General options");
	if (config->readBoolEntry("first", true)){
		/** Display a messagebox and execute the preferencesDialog */
		KMessageBox::information(0, i18n(\
			"You don't seem to have a valid configuration.\
That's ok if you start OpenOffice.org Quickstarter for the first time.\n\n\
The configuration-dialog will now be called so that you can enter the required data.\n\
You can access the dialog any time later by selecting \"Configure OpenOffice.org Quickstarter...\" in the context-menu."),\
i18n("No valid config found"));
		preferencesDialog->exec();
	}else{
		rereadConfig();
	}

	/** Create the fileOpen-dialog */
	config->setGroup("General options");
	open = new KFileDialog(config->readEntry("documents"), 0, this, 0, true);
	open->setMode(4+8+16);
	open->setFilter(\
"*.sx?|OpenOffice.org Document\n\
*.sxc|OpenOffice.org Calc Document\n\
*.sxd|OpenOffice.org Draw Document\n\
*.sxi|OpenOffice.org Impress Document\n\
*.sxg|OpenOffice.org Global Document\n\
*.sxm|OpenOffice.org Math Document\n\
*.sxw|OpenOffice.org Writer Document\n\
*.sd?|Star Office 5.x Document\n\
*.sdc|Star Office 5.x StarCalc Document\n\
*.sda|Star Office 5.x StarDraw or StarImpress Document\n\
*.sgl|Star Office 5.x StarWriter Globaldocument\n\
*.sdw|Star Office 5.x StarWriter Document\n\
*.html|HTML Document\n\
*.doc|Microsoft Word Document\n\
*.xls|Microsoft Excel Document\n\
*.ppt|Microsoft PowerPoint Document\n\
*.txt|Simple ASCII-text file\n\
*.csv|Simple Comma seperated table\n\
*|All files");

	instance << soffice << "-quickstart";

	/** Check if the quickstart should be enabled */
	if (quickstartEnabled){
		/** Make sure that instance is automatically restarted when it gets closed */
		connect(&instance, SIGNAL (processExited(KProcess*)), SLOT (restartOffice()));
		/** Try to start instance for the first time */
		restartOffice();
	}
}

OooQs::~OooQs()
{
}

/** Opens a KFileDialog that let's you choose some files you want to open. */
void OooQs::fileOpen(){
	QStringList fileList;
	KProcess* process = new KProcess;
	if (open->exec()){
		fileList = open->selectedFiles();
		*process << soffice << fileList;
		process->start(KProcess::DontCare);
		delete process;
		for ( QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it ) {
			recent->addURL(*it);
		}
		recent->saveEntries(config);
		config->sync();
	}
}
/** Opens the selected file. */
void OooQs::openRecent(const KURL& url){
	startInstance(url.url());
}
/** Executes the about-dialog. */
void OooQs::showAbout(){
	aboutDialog->exec();
}
/** Starts OpenOffice.org Calc with a new file */
void OooQs::startCalc(){
	startInstance("private:factory/scalc");
}
/** Starts OpenOffice.org Draw with a new file */
void OooQs::startDraw(){
	startInstance("private:factory/sdraw");
}
/** Starts OpenOffice.org Impress with a new file */
void OooQs::startImpress(){
	startInstance("private:factory/simpress");
}
/** Starts OpenOffice.org Global with a new file */
void OooQs::startGlobal(){
	startInstance("private:factory/swriter/GlobalDocument");
}
/** Starts OpenOffice.org Math with a new file */
void OooQs::startMath(){
	startInstance("private:factory/smath");
}
/** Starts OpenOffice.org Writer with a new file */
void OooQs::startWriter(){
	startInstance("private:factory/swriter");
}
/** Starts OpenOffice.org Web with a new file */
void OooQs::startWeb(){
	startInstance("private:factory/swriter/web");
}

/** Restarts OpenOffice if removed from memory. */
void OooQs::restartOffice(){
//	if (!checkProcess()){
		instance.start();
//	}
}

/** Deletes the process. */
void OooQs::delProcess(KProcess* process){
	delete process;
}
/** Call this to open the preferences-dialog. */
void OooQs::openPreferences(){
	preferencesDialog->exec();
}
/** Should be called whenever the configuration changes. */
void OooQs::changedConfig(){
	config->setGroup("General options");
	config->writeEntry("first", false);
	config->writeEntry("exec", preferencesDialog->binary->url());
	config->writeEntry("quickstart", preferencesDialog->enableQuick->isChecked());
	config->writeEntry("documents", preferencesDialog->documents->url());
	config->writeEntry("maxRecentFiles", preferencesDialog->maxEntries->value());
	config->setGroup("Modules");
	config->writeEntry("calc", preferencesDialog->calc->isChecked());
	config->writeEntry("draw", preferencesDialog->draw->isChecked());
	config->writeEntry("impress", preferencesDialog->impress->isChecked());
	config->writeEntry("global", preferencesDialog->global->isChecked());
	config->writeEntry("math", preferencesDialog->math->isChecked());
	config->writeEntry("web", preferencesDialog->web->isChecked());
	config->writeEntry("writer", preferencesDialog->writer->isChecked());
	config->writeEntry("open", preferencesDialog->open->isChecked());
	config->writeEntry("recent", preferencesDialog->recent->isChecked());
	config->setGroup("Additional");
	config->writeEntry("stdCalc", preferencesDialog->stdCalc->isChecked());
	config->writeEntry("stdDraw", preferencesDialog->stdDraw->isChecked());
	config->writeEntry("stdImpress", preferencesDialog->stdImpress->isChecked());
	config->writeEntry("stdGlobal", preferencesDialog->stdGlobal->isChecked());
	config->writeEntry("stdMath", preferencesDialog->stdMath->isChecked());
	config->writeEntry("stdWeb", preferencesDialog->stdWeb->isChecked());
	config->writeEntry("stdWriter", preferencesDialog->stdWriter->isChecked());
	config->writeEntry("stdOpen", preferencesDialog->stdOpen->isChecked());
	config->writeEntry("stdLast", preferencesDialog->stdLast->isChecked());
	config->writeEntry("nothing", preferencesDialog->nothing->isChecked());
	config->sync();
	rereadConfig();
}
/** Call this to (re-)load the config from disk */
void OooQs::rereadConfig(){
	QString tempURL = "";
	/** Reparse the configuration to be shure that the data we use is up to date */
	config->reparseConfiguration();
	/** Set the values */
	config->setGroup("General options");
	soffice = config->readEntry("exec");
	if (quickstartEnabled != config->readBoolEntry("quickstart")){
		if (quickstartEnabled){
			quickstartEnabled=false;
		}else{
			restartOffice();
			quickstartEnabled=true;
		}
	}
	if (open){
		tempURL = config->readEntry("documents", "not_there");
		if (tempURL != "not_there"){
			open->setURL(tempURL);
		}
	}
	recent->setMaxItems(config->readUnsignedNumEntry("maxRecentFiles"));

	/** Delete the menu */
	calc->unplug(contextMenu());
	draw->unplug(contextMenu());
	impress->unplug(contextMenu());
	global->unplug(contextMenu());
	math->unplug(contextMenu());
	web->unplug(contextMenu());
	writer->unplug(contextMenu());
	file->unplug(contextMenu());
	recent->unplug(contextMenu());
	preferences->unplug(contextMenu());
	about->unplug(contextMenu());
	sep[0]->unplug(contextMenu());
	sep[1]->unplug(contextMenu());
	sep[2]->unplug(contextMenu());
	sep[3]->unplug(contextMenu());
/*	if (editMenuFirst){
		contextMenu()->removeItemAt(1);
	}*/

	int counter = 1;
	config->setGroup("Modules");
	if (config->readBoolEntry("open")){
		file->plug(contextMenu(), counter);
		counter++;
		if (config->readBoolEntry("recent")){
			recent->plug(contextMenu(), counter);
			counter++;
		}
		sep[0]->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("calc")){
		calc->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("draw")){
		draw->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("impress")){
		impress->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("global")){
		global->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("math")){
		math->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("web")){
		web->plug(contextMenu(), counter);
		counter++;
	}
	if (config->readBoolEntry("writer")){
		writer->plug(contextMenu(), counter++);
		counter++;
	}
	if (counter != 1){
		sep[1]->plug(contextMenu(), counter);
		counter++;
	}
	preferences->plug(contextMenu(), counter);
	counter++;
	sep[2]->plug(contextMenu(), counter);
	counter++;
	about->plug(contextMenu(), counter);
	if (editMenuFirst){
		counter++;
		sep[3]->plug(contextMenu(), counter);
		editMenuFirst = false;
	}

	calc->setEnabled(config->readBoolEntry("calc"));
	draw->setEnabled(config->readBoolEntry("draw"));
	impress->setEnabled(config->readBoolEntry("impress"));
	global->setEnabled(config->readBoolEntry("global"));
	math->setEnabled(config->readBoolEntry("math"));
	web->setEnabled(config->readBoolEntry("web"));
	writer->setEnabled(config->readBoolEntry("writer"));
	file->setEnabled(config->readBoolEntry("open"));
	if (file->isEnabled()){
		recent->setEnabled(config->readBoolEntry("recent"));
	}else{
		recent->setEnabled(false);
	}

	setDoubleClickAction();
	
	/** Set the widgets in the preferences dialog to the current values */
	config->setGroup("General options");
	preferencesDialog->binary->setURL(soffice);
	preferencesDialog->enableQuick->setChecked(quickstartEnabled);
	preferencesDialog->maxEntries->setValue(recent->maxItems());
	preferencesDialog->documents->setURL(config->readEntry("documents"));
	config->setGroup("Modules");
	preferencesDialog->calc->setChecked(calc->isEnabled());
	preferencesDialog->draw->setChecked(draw->isEnabled());
	preferencesDialog->impress->setChecked(impress->isEnabled());
	preferencesDialog->global->setChecked(global->isEnabled());
	preferencesDialog->math->setChecked(math->isEnabled());
	preferencesDialog->web->setChecked(web->isEnabled());
	preferencesDialog->writer->setChecked(writer->isEnabled());
	preferencesDialog->open->setChecked(file->isEnabled());
	preferencesDialog->recent->setChecked(recent->isEnabled());
	config->setGroup("Additional");
	preferencesDialog->stdCalc->setChecked(config->readBoolEntry("stdCalc"));
	preferencesDialog->stdDraw->setChecked(config->readBoolEntry("stdDraw"));
	preferencesDialog->stdImpress->setChecked(config->readBoolEntry("stdImpress"));
	preferencesDialog->stdGlobal->setChecked(config->readBoolEntry("stdGlobal"));
	preferencesDialog->stdMath->setChecked(config->readBoolEntry("stdMath"));
	preferencesDialog->stdWeb->setChecked(config->readBoolEntry("stdWeb"));
	preferencesDialog->stdWriter->setChecked(config->readBoolEntry("stdWriter"));
	preferencesDialog->stdOpen->setChecked(config->readBoolEntry("stdOpen"));
	preferencesDialog->stdLast->setChecked(config->readBoolEntry("stdLast"));
	preferencesDialog->nothing->setChecked(config->readBoolEntry("nothing"));
}
/** Call this to start OpenOffice.org with the given parameters */
void OooQs::startInstance(QString param){
	KProcess* process = new KProcess;
	*process << soffice << param;
	process->start(KProcess::DontCare);
	connect( process, SIGNAL (processExited(KProcess*)), SLOT (delProcess(KProcess*)));
}
/** Check if a process, possibly started by instance, exists.
Possibly only working on Linux. */
bool OooQs::checkProcess(){
	KProcIO* ps = new KProcIO;
	QString retval;
	//*ps << "ps" << "h" << "-C" << soffice;
	*ps << "echo" << "bla";
	ps->start(KProcess::NotifyOnExit, KProcess::AllOutput);
	ps->readln(retval, true);
	return true;
}

void OooQs::mousePressEvent(QMouseEvent* event){
	if (event->type() == QEvent::MouseButtonDblClick){
		emit doubleClick();
	}
	if (event->type() == QEvent::MouseButtonPress){
		if (event->button() == QMouseEvent::RightButton){
			contextMenu()->exec(QCursor::pos());
		}
	}
}
/** Open the last file the user opened using the open-dialog. */
void OooQs::openLast(){
	QString file;
	config->setGroup("RecentFiles");
	file = config->readEntry("File1");
	if (file != ""){
		startInstance(file);
	}else{
		fileOpen();
	}
}
/** Set the current DoubleClickAction */
void OooQs::setDoubleClickAction(){
	disconnect(SIGNAL (doubleClick()));
	QToolTip::remove(this);
	config->setGroup("Additional");
	if (config->readBoolEntry("stdCalc")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startCalc()));
		QToolTip::add(this, calc->text());
		return;
	}
	if (config->readBoolEntry("stdDraw")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startDraw()));
		QToolTip::add(this, draw->text());
		return;
	}
	if (config->readBoolEntry("stdImpress")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startImpress()));
		QToolTip::add(this, impress->text());
		return;
	}
	if (config->readBoolEntry("stdGlobal")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startGlobal()));
		QToolTip::add(this, global->text());
		return;
	}
	if (config->readBoolEntry("stdMath")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startMath()));
		QToolTip::add(this, math->text());
		return;
	}
	if (config->readBoolEntry("stdWeb")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startWeb()));
		QToolTip::add(this, web->text());
		return;
	}
	if (config->readBoolEntry("stdWriter")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (startWriter()));
		QToolTip::add(this, writer->text());
		return;
	}
	if (config->readBoolEntry("stdOpen")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (fileOpen()));
		QToolTip::add(this, file->text());
		return;
	}
	if (config->readBoolEntry("stdLast")){
		connect(this, SIGNAL (doubleClick()), this, SLOT (openLast()));
		config->setGroup("RecentFiles");
		if (config->readEntry("File1") != ""){
			QToolTip::add(this, i18n("Open") + " " + config->readEntry("File1"));
		}else{
			QToolTip::add(this, "<" + i18n("History empty") + ">");
		}
		return;
	}
	QToolTip::add(this, i18n("OpenOffice.org Quickstarter"));
}
/** Call this to clear the list of recent files. */
void OooQs::clearURLList(){
	recent->clearURLList();
	for (int i=recent->maxItems(); i > 0; i--){
		recent->addURL("");
	}
	recent->saveEntries(config);
	config->sync();
}
