/***************************************************************************
                          oooqs.h  -  description
                             -------------------
    begin                : Mon Apr 29 17:48:15 CEST 2002
    copyright            : (C) 2002 by Christian Nitschkowski
    email                : segfault_ii@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef OOOQS_H
#define OOOQS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
//#include <ksystemtray.h>
#include <kdockwindow.h>
#include <kpixmap.h>
#include <kaction.h>
#include <kdialog.h>
#include <kprocess.h>
#include <prefdialogimpl.h>
#include <kconfig.h>
#include <kurl.h>
#include <kfiledialog.h>

/** OooQs is the base class of the project */
//class OooQs : public KSystemTray
class OooQs : public KDockWindow
{
  Q_OBJECT 
  public:
    /** construtor */
    OooQs(KDialog* d, QWidget* parent=0, const char *name=0);
    /** destructor */
    ~OooQs();
private: // Private attributes
    /** Saves the OpenOffice.org icon */
    KPixmap icon;
    /** The actionCollection containing the contextmenu-entrys */
    KActionCollection actionCollection;
    /** The action that calls OpenOffice.org Calc. */
    KAction* calc;
    /** The action that calls OpenOffice.org Draw. */
    KAction* draw;
    /** The action that calls OpenOffice.org Impress. */
    KAction* impress;
    /** The action that calls OpenOffice.org Global. */
    KAction* global;
    /** The action that calls OpenOffice.org Math. */
    KAction* math;
    /** The action that calls OpenOffice.org Web. */
    KAction* web;
    /** The action that calls OpenOffice.org Writer. */
    KAction* writer;
    /** The action that calls the about-dialog. */
    KAction* about;
    /** The action that calls the openfile-dialog. */
    KAction* file;
    /** The action that calls the openfile-dialog. */
    KRecentFilesAction* recent;
    /** The action that calls the preferences-dialog. */
    KAction* preferences;
    /** The quit-item */
    KAction* quit;
    /** The about-dialog */
    KDialog* aboutDialog;
    /** Some separators */
    KActionSeparator* sep[4];
  /** The KProcess that starts and control OpenOffice.org in quickstart-mode. */
  KProcess instance;
  /** Saves the path to the soffice startscript/executable */
  QString soffice;
  /** Saves if the quickstart should be enabled */
  bool quickstartEnabled;
  /** The dialog where we'll configure the few settings available for oooqs. */
  prefDialogImpl* preferencesDialog;
  /** The configuration used by oooqs */
  KConfig* config;
    /** The fileOpen-dialog */
    KFileDialog* open;
    /** Saves if the automatically inserted separator has to be removed */
    bool editMenuFirst;
private slots: // Private slots
  /** Opens a KFileDialog that let's you choose some files you want to open. */
  void fileOpen();
  /** Opens the selected file. */
  void openRecent(const KURL& url);
  /** Starts OpenOffice.org Calc with a new file */
  void startCalc();
  /** Starts OpenOffice.org Draw with a new file */
  void startDraw();
  /** Starts OpenOffice.org Impress with a new file */
  void startImpress();
  /** Starts OpenOffice.org with a new global file */
  void startGlobal();
  /** Starts OpenOffice.org Math with a new file */
  void startMath();
  /** Starts OpenOffice.org Web with a new file */
  void startWeb();
  /** Starts OpenOffice.org Writer with a new file */
  void startWriter();
  /** Executes the about-dialog. */
  void showAbout();
  /** Restarts OpenOffice if removed from memory. */
  void restartOffice();
  /** Delete the process. */
  void delProcess(KProcess* process);
  /** Call this to open the preferences-dialog. */
  void openPreferences();
  /** Should be called whenever the configuration changes. */
  void changedConfig();
  /** Call this on doubel clicks on the icon. */
  void mousePressEvent(QMouseEvent*);
  /** Open the last file the user opened using the open-dialog. */
  void openLast();
private:
    /** Call this to (re-)load the config from disk */
    void rereadConfig();
    /** Call this to start OpenOffice.org with the given parameters */
    void startInstance(QString param);
    /** Check if a process, possibly started by instance, exists.
    Possibly only working on Linux. */
    bool checkProcess();
    /** Set the current DoubleClickAction */
    void setDoubleClickAction();
signals:
    /** Emit when a double click was registered */
    void doubleClick();
public slots: // Public slots
  /** Call this to clear the list of recent files. */
  virtual void clearURLList();
};

#endif
